package com.gic.haoban.app.daily.mobile.web.utils;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.common.utils.AppRequestUtil;
import com.task.allocation.util.MemCachedUtil;
import net.sf.json.JSONObject;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;

public class AppUtil {
	//static MemCachedInstance memCachedInstance = MemCachedInstance.getInstance(GlobalVar.getMemCachedIp(), GlobalVar.getMemCachedPort());
	
	private static Logger logger = LogManager.getLogger(AppUtil.class);
	/**
	 * 创建4位验证码
	 * @return
	 */
	public static String createSMSCode() {
		Random random = new Random();
		StringBuilder result = new StringBuilder();
		for (int i = 0; i < 4; i++) {
			result.append(random.nextInt(10));
		}
		return result.toString();
	}
	
	/**
	 * 门店APP缓存验证码
	 * @param phoneNumber
	 * @param smscode
	 */
	public static void setSMScode(String phoneNumber, String smScode) {
		 Map<String, Object> map=new HashMap<String, Object>();
			map.put(phoneNumber, smScode);
			map.put("timeStamp", System.currentTimeMillis());
			String value= JSON.toJSONString(map);
			MemCachedUtil.setValue(phoneNumber+"_smsCode", value, 10*60*1000);
	}
	
	/**
	 * 验证码验证
	 * @param phoneNumber
	 * @param smscode
	 * @return
	 */
	public static boolean trueCode(String phoneNumber, String smscode) {
		
		Object storeCode = MemCachedUtil.getValue(phoneNumber+"_smsCode");
		Map<String, String> map = new HashMap<String, String>();
		if(AppRequestUtil.notEmpty(storeCode)){
			map = toMap(storeCode);
		}
		logger.info(phoneNumber+"缓存验证码为："+map.get(phoneNumber)+"验证验证码为："+smscode);
		if(map.containsKey(phoneNumber) && map.get(phoneNumber).equals(smscode)){
			return true;
		}
		return false;
	}
	
	/**
	 * 对象转map
	 * @param object
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	public static Map<String, String> toMap(Object object) {
	    Map<String, String> data = new HashMap<String, String>();
	    // 将json字符串转换成jsonObject
	    JSONObject jsonObject = JSONObject.fromObject(object);
	    Iterator ite = jsonObject.keys();
	    // 遍历jsonObject数据,添加到Map对象
	    while (ite.hasNext()) {
	        String key = ite.next().toString();
	        String value = jsonObject.get(key).toString();
	        data.put(key, value);
	    }
	    return data;
	}
	
	/**
	 * 百分比格式化
	 * @param numerator		分子
	 * @param denominator	分母
	 * @param maximum		小数点保留几位
	 * @return
	 */
	public static String returnPercent(float ft,int maximum){
		NumberFormat numberFormat = NumberFormat.getPercentInstance();  
		numberFormat.setMaximumFractionDigits(0);
		return numberFormat.format(ft);
	}
	
	/**
	 * token生成
	 * @param phoneNumber
	 * @return
	 */
	public static String creatToken(String phoneNumber){
		return phoneNumber+ToolUtil.randomUUID();
	}
	
	/**
	 * 门店APP缓存token
	 * @param phoneNumber
	 * @param token
	 */
	public static void setToken(String phoneNumber, String token) {
		 Map<String, Object> map=new HashMap<String, Object>();
			map.put(phoneNumber, token);
			map.put("timeStamp", System.currentTimeMillis());
			String value= JSON.toJSONString(map);
			logger.info(">>>>>>>>>>>>>>>>生成token为："+token);
			MemCachedUtil.setValue(phoneNumber+"_appToken", value, 0);
	}
	
	/**
	 * 删除并建立新token
	 * @param newPhoneNumber	新手机号
	 * @param phoneNumber		原手机号
	 * @param token				token
	 */
	public static void delAndCreateToken(String phoneNumber, String newPhoneNumber, String token) {
		// 清除原TOKEN
		MemCachedUtil.delete(phoneNumber+"_appToken");
		Map<String, Object> map=new HashMap<String, Object>();
		map.put(newPhoneNumber, token);
		map.put("timeStamp", System.currentTimeMillis());
		String value= JSON.toJSONString(map);
		MemCachedUtil.setValue(newPhoneNumber+"_appToken", value, 0);
	}
	
	/**
	 * 删除TOKEN
	 * @param phoneNumber
	 */
	public static boolean delToken(String phoneNumber) {
		return MemCachedUtil.delete(phoneNumber+"_appToken");
	}
	
	/**
	 * token校验
	 * @param phoneNumber
	 * @param token
	 * @return
	 */
	public static boolean trueToken(String phoneNumber, String token) {
		Object storeCode = MemCachedUtil.getValue(phoneNumber+"_appToken");
		Map<String, String> map = new HashMap<String, String>();
		if(AppRequestUtil.notEmpty(storeCode)){
			map = toMap(storeCode);
		}
		logger.info(">>>>>>>>>>>>>>>>>>>>>>>>>>缓存token为："+map.get(phoneNumber)+"校验token为："+token);
		if(map.containsKey(phoneNumber) && map.get(phoneNumber).equals(token) && !"".equals(map.get(phoneNumber))){
			return true;
		}
		return false;
	}

	public static String getToken(String phoneNumber) {
		Object storeCode = MemCachedUtil.getValue(phoneNumber+"_appToken");
        Map<String, String> map = new HashMap<String, String>();
		if(AppRequestUtil.notEmpty(storeCode)){
            map = toMap(storeCode);
			return map.get(phoneNumber);
		}
		return null;
	}
	
	/**
	 * 系统当前时间
	 * @return
	 */
	public static String sysDate(){
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//设置日期格式
		return df.format(new Date());// new Date()为获取当前系统时间
	}
	
	/**
	 * 缓存极光消息推送设备ID
	 * @param clerkId
	 * @param registrationId
	 */
	public static void setRegistrationId(String clerkId, String registrationId) {
		Map<String, Object> map=new HashMap<String, Object>();
		map.put(clerkId, registrationId);
		map.put("timeStamp", System.currentTimeMillis());
		String value= JSON.toJSONString(map);
		logger.info(clerkId+"缓存消息推送ID为："+registrationId);
		MemCachedUtil.setValue(clerkId+GlobalInfo.SEND_REGISTRATIONID, value, 0);
	}
	
	/**
	 * 获取极光消息推送设备ID
	 * @param clerkId
	 * @return
	 */
	public static String getRegistrationId(String clerkId){
		Object registrationId = MemCachedUtil.getValue(clerkId+GlobalInfo.SEND_REGISTRATIONID);
		Map<String, String> map = new HashMap<String, String>();
		if(AppRequestUtil.notEmpty(registrationId)){
			map = toMap(registrationId);
		}
		if(map.containsKey(clerkId) && !"".equals(map.get(clerkId))){
			return map.get(clerkId);
		}
		logger.info(">>>>>>>>>>>>>>>>>>>>>>缓存清空成功"+clerkId);
		return null;
	}
	
	/**
	 * 清除极光消息推送设备ID
	 * @param clerkId
	 * @return
	 */
	public static void delRegistrationId(String clerkId){
		logger.info(">>>>>>>>>>>>>>>>>>>>>>>>>>>切换品牌清空推送设备ID"+clerkId);
		MemCachedUtil.delete(clerkId+GlobalInfo.SEND_REGISTRATIONID);
		getRegistrationId(clerkId);
	}
	
	/**
	 * DOUBLE小数点精确
	 * @param db		DOUBLE原数据
	 * @param count		小数点后位数
	 * @return
	 */
	public static double formatterDouble(Double db, int count){
		BigDecimal b= new BigDecimal(db);  
		double f1 = b.setScale(count, BigDecimal.ROUND_HALF_UP).doubleValue(); 
		return f1;
	}

	/**
	 * 小叔点保留两位
	 * @param value
	 * @return
	 */
	public static String changePrecent(double value,int count) {
		BigDecimal bd = new BigDecimal(value);
		BigDecimal multiply = bd.multiply(new BigDecimal(100));
		multiply = multiply.setScale(count, BigDecimal.ROUND_HALF_UP);
		return multiply.toString()+"%";
	}
}
